@@ -16,6 +16,14 @@ del environ, path
 # SWIG-generated
 from . import ccllib as lib
 
+# monkey patch for isitgr and fast-pt if Numpy>=1.24
+from packaging.version import parse
+import numpy as np
+if parse(np.__version__) >= parse('1.24'):
+    np.int = int
+del parse
+del np
+
 # Errors
 from .errors import (
     CCLError,
