@@ -62,8 +62,8 @@ static ccl_cosmology *create_w0eff_cosmo(double w0eff, ccl_cosmology *cosmo, int
     cosmo->params.Neff, mnu, cosmo->params.N_nu_mass,
     w0eff, 0, cosmo->params.h, norm_pk,
     cosmo->params.n_s, cosmo->params.bcm_log10Mc, cosmo->params.bcm_etab,
-    cosmo->params.bcm_ks, cosmo->params.mu_0, cosmo->params.sigma_0, 
-    cosmo->params.c1_mg, cosmo->params.c2_mg, cosmo->params.lambda_mg, 
+    cosmo->params.bcm_ks, cosmo->params.mu_0, cosmo->params.sigma_0,
+    cosmo->params.c1_mg, cosmo->params.c2_mg, cosmo->params.lambda_mg,
     cosmo->params.nz_mgrowth,
     cosmo->params.z_mgrowth, cosmo->params.df_mgrowth, status);
 
@@ -236,7 +236,7 @@ static double rsigma_func(double rsigma, void *p) {
   int gsl_status;
 
   lnkmin = hfd->plin->lkmin;
-  lnkmax = hfd->plin->lkmax;
+  lnkmax = fmax(hfd->plin->lkmax, log(30/rsigma));
   hfd->r = rsigma;
   hfd->r2 = rsigma * rsigma;
   F.function = &gauss_norm_int_func;
@@ -257,8 +257,12 @@ static double rsigma_func(double rsigma, void *p) {
   return result - 1.0;
 }
 
+static double lnrsigma_func(double lnrsigma, void *p) {
+  return rsigma_func(exp(lnrsigma), p);
+}
+
 static double get_rsigma(double a, struct hf_int_data data) {
-  double rsigma, rlow = 1e-2, rhigh = 1e2;
+  double rsigma, rlow = log(1e-64), rhigh = log(1e16);
   double flow, fhigh;
   int itr, max_itr = 1000, gsl_status;
   const gsl_root_fsolver_type *T;
@@ -266,13 +270,13 @@ static double get_rsigma(double a, struct hf_int_data data) {
   gsl_function F;
 
   data.a = a;
-  F.function = &rsigma_func;
+  F.function = &lnrsigma_func;
   F.params = &data;
 
   // we have to bound the root, otherwise return -1
   // we will fiil in any -1's in the calling routine
-  flow = rsigma_func(rlow, &data);
-  fhigh = rsigma_func(rhigh, &data);
+  flow = lnrsigma_func(rlow, &data);
+  fhigh = lnrsigma_func(rhigh, &data);
   if (flow * fhigh > 0) {
     return -1;
   }
@@ -310,6 +314,7 @@ static double get_rsigma(double a, struct hf_int_data data) {
       *(data.status) |= gsl_status;
     }
   }
+  rsigma = exp(rsigma);
 
   return rsigma;
 }
@@ -564,7 +569,7 @@ halofit_struct* ccl_halofit_struct_new(ccl_cosmology *cosmo,
 
     for (i=0; i<n_a; ++i) {
       vals[i] = get_rsigma(a_vec[i], data);
-      if (*status != 0) {
+      if ((*status != 0) || (vals[i] <= 0)) {
         *status = CCL_ERROR_ROOT;
         ccl_cosmology_set_status_message(
           cosmo,
@@ -573,51 +578,6 @@ halofit_struct* ccl_halofit_struct_new(ccl_cosmology *cosmo,
         break;
       }
     }
-
-    // now go backwards and fill any -1's
-    // one must work, so set an error if not
-    if (vals[n_a-1] == -1) {
-      *status = CCL_ERROR_ROOT;
-      ccl_cosmology_set_status_message(
-        cosmo,
-        "ccl_halofit.c: ccl_halofit_struct_new(): "
-        "could not solve for non-linear scale for halofit at least once\n");
-    }
-    if (*status == 0) {
-      // linearly set rsigma to a very small number at high-z
-      double min_a = -1, max_a = -1;
-      double max_val = -1;
-      double w;
-
-      // first non -1 value and scale factor of last -1 value
-      for (i=1; i<n_a; ++i) {
-        if (vals[i] != -1) {
-          max_a = a_vec[i];
-          max_val = vals[i];
-          break;
-        }
-      }
-      // scale factor of first -1 value
-      for (i=0; i<n_a; ++i) {
-        if (vals[i] == -1) {
-          min_a = a_vec[i];
-          break;
-        }
-      }
-
-      if (min_a != -1) {
-        // at least one value is -1 so set the zeroth value
-        vals[0] = 1e-6;
-
-        // interp any values that remain -1
-        for(i=1; i<n_a-1; ++i) {
-          if (vals[i] == -1) {
-            w = (a_vec[i] - min_a) / (max_a - min_a);
-            vals[i] = w * max_val + (1.0 - w) * vals[0];
-          }
-        }
-      }
-    }
   }
 
   // spline the non-linear scales
@@ -697,7 +657,8 @@ halofit_struct* ccl_halofit_struct_new(ccl_cosmology *cosmo,
 
       gsl_status = gsl_integration_cquad(
         &F,
-        lnkmin, lnkmax,
+        lnkmin,
+        fmax(lnkmax, log(30/rsigma)),
         0.0, cosmo->gsl_params.INTEGRATION_SIGMAR_EPSREL,
         workspace, &result, NULL, NULL);
 
@@ -761,7 +722,8 @@ halofit_struct* ccl_halofit_struct_new(ccl_cosmology *cosmo,
 
       gsl_status = gsl_integration_cquad(
         &F,
-        lnkmin, lnkmax,
+        lnkmin,
+        fmax(lnkmax, log(30/rsigma)),
         0.0, cosmo->gsl_params.INTEGRATION_SIGMAR_EPSREL,
         workspace, &result, NULL, NULL);
 
